
using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading;
using ClarionSimulation;
using WorldServerLibrary;
using WorldServerLibrary.Model;
using WorldServerLibrary.Exceptions;

namespace ClarionDEMO
{
	class MainClass
	{
		#region properties
		private WorldServer worldServer = null;
        private ClarionAgent agent;
        String creatureId = String.Empty;
        String creatureName = String.Empty;
		double prad = 0;
		private IList<Thing> objetos = null;
		#endregion

		#region constructor
		public MainClass() {
			Console.WriteLine ("Clarion Demo V0.4");
			agent = new ClarionAgent();
            agent.OnNewVisualSensorialInformation += new InputVisualSensorialInformationEventHandler(agent_OnNewVisualSensorialInformation);
            agent.OnNewExternalActionSelected += new OutputActionChunkEventHandler(agent_OnNewExternalActionSelected);
			try
            {
                worldServer = new WorldServer("localhost", 4011);
                String message = worldServer.Connect();

                if (worldServer != null && worldServer.IsConnected)
                {
                    Console.Out.WriteLine ("[SUCCESS] " + message + "\n");
					worldServer.SendWorldReset();
                    worldServer.NewCreature(80, 80, 0, out creatureId, out creatureName);
					worldServer.SendCreateLeaflet();

                    worldServer.NewBrick(4, 747, 2, 800, 567);
                    worldServer.NewBrick(4, 50, -4, 747, 47);
                    worldServer.NewBrick(4, 49, 562, 796, 599);
                    worldServer.NewBrick(4, -2, 6, 50, 599);

					//adiciona os objetos no mundo
					/*
							---------
							|*		|
							|		|
							|		|
							---------
					*(X0,Y0)
					*/

					//Adiciona 20 jóias de cores aleatórias no ambiente
					Random numRand = new Random();
					for (int i=0;i<=20;i++){
						int tipo=numRand.Next(0,6);
						int x=numRand.Next(100,700);
						int y=numRand.Next(100,500);
						worldServer.NewJewel(tipo,x,y);
					}
						
					if (!String.IsNullOrWhiteSpace(creatureId))
                    {
                        worldServer.SendStartCamera(creatureId);
                        worldServer.SendStartCreature(creatureId);
                    }

                    Console.Out.WriteLine("Creature created with name: " + creatureId + "\n");
                    agent.Run();
					Console.Out.WriteLine("Running Simulation ...\n");
                }
            }
            catch (WorldServerInvalidArgument invalidArtgument)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Invalid Argument: {0}\n", invalidArtgument.Message));
            }
            catch (WorldServerConnectionError serverError)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Is is not possible to connect to server: {0}\n", serverError.Message));
            }
            catch (Exception ex)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Unknown Error: {0}\n", ex.Message));
            }
		}
		#endregion

		#region Methods
		public static void Main (string[] args)	{
			new MainClass();
		}
		IList<Thing> agent_OnNewVisualSensorialInformation ()
					
        {

            IList<Thing> response = null;

            if (worldServer != null && worldServer.IsConnected)
            {
                response = worldServer.SendGetCreatureState(creatureName);
				prad = (Math.PI / 180) * response.First().Pitch;
			    while (prad > Math.PI) prad -= 2 * Math.PI;
			    while (prad < - Math.PI) prad += 2 * Math.PI;
				//Sack s = worldServer.SendGetSack(creatureId);
				//s.print();
            }
			objetos = response;
            return response;
        }



        void agent_OnNewExternalActionSelected(ClarionAgentActionType externalAction)
        {   Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");
            if (worldServer != null && worldServer.IsConnected)
            {
				Thing proximo = null;
				Double menorDist = Double.MaxValue;
				Console.WriteLine ("Ação Selecionada " + Convert.ToString (externalAction));
					switch (externalAction)
                	{
                    	case ClarionAgentActionType.DO_NOTHING:
                        // Do nothing as the own value says
                        	break;
                    	case ClarionAgentActionType.ROTATE_CLOCKWISE:
					    	double newprad = prad + Math.PI/1000;
					    	while (newprad > Math.PI) newprad -= 2 * Math.PI;
			            	while (newprad < - Math.PI) newprad += 2 * Math.PI;
                        	worldServer.SendSetAngle(creatureId, 0, 0, newprad);
                        	break;
						case ClarionAgentActionType.GOTOJEWEL:
							Creature creature = null;
							foreach (Thing item in objetos) {
								if (item.CategoryId == Thing.CATEGORY_CREATURE) {
									creature = (Creature)item;
								}	
							}
						
							menorDist = 10000;
							foreach (Thing item in objetos) {
								if ((item.CategoryId == Thing.CATEGORY_JEWEL) && item.DistanceToCreature <= menorDist) {
									menorDist = item.DistanceToCreature;
									proximo = item;
								}
							}
							if (proximo != null) {
								worldServer.SendSetGoTo (creatureId, 1.0, 1.0, proximo.X1, proximo.Y1);
							}
							break;
						case ClarionAgentActionType.EXPLORE:
							newprad = prad + Math.PI/1000;
							while (newprad > Math.PI) newprad -= 2 * Math.PI;
							while (newprad < - Math.PI) newprad += 2 * Math.PI;
							//worldServer.SendSetTurn (creatureId, 1.0, 2.0, 1.0);
							worldServer.SendSetAngle(creatureId, 0, 0, newprad);
							break;
                    	
						case ClarionAgentActionType.GET_JEWEL:
							proximo = null;
							menorDist = 10000;
							foreach (Thing item in objetos) {
								if ((item.CategoryId == Thing.CATEGORY_JEWEL) && item.DistanceToCreature <= menorDist) {
									menorDist = item.DistanceToCreature;
									proximo = item;
								}
							}
							if (proximo != null) {
								if (menorDist <= 28) {
									worldServer.SendSackIt (creatureId, proximo.Name);
								}
							}
							//#Atualiza a lista de leaflet somente após ter pego o item
							Sack s = worldServer.SendGetSack (creatureId);
							s.print ();
							break;
						case ClarionAgentActionType.GO_AHEAD:
							worldServer.SendSetAngle(creatureId, 1.0, 1.0, prad);
						 	break;
						default:
							break;
					}						
			}
		}

        #endregion
	}
	
	
}
